Package: LFApp
Version: 1.4.2
Date: 2026-05-05
Title: Shiny Apps for Lateral Flow Assays
Authors@R: c(person("Filip", "Paskali", role = c("aut", "cre"),
                  email = "Filip.Paskali@gmail.com",
                  comment = c(ORCID = "0000-0002-9647-6294")),
             person("Weronika", "Schary", role = "aut",
                  email = "Weronika.Schary@hs-furtwangen.de", 
                  comment = c(ORCID = "0000-0002-7229-316X")),
             person("Matthias", "Kohl", role = "aut",
                  email = "Matthias.Kohl@stamats.de", 
                  comment = c(ORCID = "0000-0001-9514-8910")))
Author: Filip Paskali [aut, cre] (ORCID:
    <https://orcid.org/0000-0002-9647-6294>),
  Weronika Schary [aut] (ORCID: <https://orcid.org/0000-0002-7229-316X>),
  Matthias Kohl [aut] (ORCID: <https://orcid.org/0000-0001-9514-8910>)
Maintainer: Filip Paskali <Filip.Paskali@gmail.com>
Description: Shiny apps for the quantitative analysis of images from lateral flow assays (LFAs). The images are segmented and background corrected and color intensities are extracted. The apps can be used to import and export intensity data and to calibrate LFAs by means of linear, loess, or gam models. The calibration models can further be saved and applied to intensity data from new images for determining concentrations.
License: LGPL-3
Depends: R (>= 4.0.0)
Imports: stats, utils, graphics, methods, mgcv, shiny, shinyjs,
        shinythemes, shinyFiles, shinyMobile (>= 0.9), EBImage, DT,
        ggplot2, fs
Suggests: knitr, rmarkdown, remotes
VignetteBuilder: knitr
Encoding: UTF-8
URL: https://github.com/fpaskali/LFApp
BugReports: https://github.com/fpaskali/LFApp/issues
NeedsCompilation: no
Packaged: 2026-05-05 18:56:18 UTC; paf
Repository: CRAN
Date/Publication: 2026-05-05 19:20:18 UTC
Built: R 4.6.0; ; 2026-05-05 21:39:16 UTC; unix
