\name{LFApp-package}
\alias{LFApp-package}
\alias{LFApp}
\docType{package}
\title{
Shiny Apps for Lateral Flow Assays.
}
\description{
Shiny apps for the quantitative analysis of images from lateral flow assays (LFAs). 
The images are segmented and background corrected and color intensities are extracted. 
The apps can be used to import and export intensity data and to calibrate LFAs by 
means of linear, loess, or gam models. The calibration models can further be saved 
and applied to intensity data from new images for determining concentrations.
}
\details{
library(LFApp)
}
\author{ Filip Paskali, Weronika Schary, Matthias Kohl

Maintainer: Filip Paskali  \email{Filip.Paskali@gmail.com}}
%\references{
%~~ Literature or other references for background information ~~
%}
%~~ Optionally other keywords from doc/KEYWORDS, one per line
%\seealso{
%~~ Optional links to other man pages, e.g. ~~
%~~ \code{\link[<pkg>:<pkg>-package]{<pkg>}} ~~
%}
%\examples{
%~~ simple examples of the most important functions ~~
%}
\keyword{package}
